"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NitinolService = void 0;
const axios = require("axios");
const https = require("https");
const index_1 = require("../lib/lambda-logger/index");
const querystring_1 = require("querystring");
const config_1 = require("../config");
const aws_sdk_1 = require("aws-sdk");
// Call to Notification Service to send mail
class NitinolService {
    static sendEmail(templateName, toAddress, fromAddress, ccAddresses, bccAddresses, subject, templateData) {
        return __awaiter(this, void 0, void 0, function* () {
            let emailResponse;
            let accessToken = "";
            const configuration = yield (0, config_1.Configuration)();
            let getSecretValuePath = process.env.stage + "/paladin/auth-secrets";
            let data = yield getSecretValue(getSecretValuePath, this.log);
            const authPayLoad = {
                client_id: JSON.parse(data)["nitinol_client_id"],
                client_secret: JSON.parse(data)["nitinol_client_secret"],
                grant_type: "client_credentials"
            };
            let emailPayLoad;
            if (fromAddress) {
                emailPayLoad = {
                    emailTemplateName: templateName,
                    toAddress,
                    fromAddress,
                    ccAddresses,
                    bccAddresses,
                    withPayloadAsAttachment: false,
                    subject,
                    templateData,
                };
            }
            else {
                emailPayLoad = {
                    emailTemplateName: templateName,
                    toAddress,
                    ccAddresses,
                    bccAddresses,
                    withPayloadAsAttachment: false,
                    subject,
                    templateData,
                };
            }
            try {
                this.log.info("Debug:Getting access token...");
                const agent = new https.Agent({
                    rejectUnauthorized: false,
                });
                process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
                const tokenResponse = yield axios.post(configuration.emailConfig.emailLogin, (0, querystring_1.stringify)(authPayLoad), {
                    headers: {
                        "Content-Type": "application/x-www-form-urlencoded",
                        Host: configuration.emailConfig.auth.host,
                    },
                }, { httpsAgent: agent });
                if (tokenResponse && tokenResponse.data) {
                    this.log.info("Start: Sending Email with payload...", emailPayLoad);
                    accessToken = tokenResponse.data.access_token;
                    try {
                        const response = yield axios.post(configuration.emailConfig.emailService, emailPayLoad, {
                            headers: {
                                "Content-Type": "application/json",
                                Authorization: "Bearer " + accessToken,
                                Host: configuration.emailConfig.auth.host,
                            },
                        }, { httpsAgent: agent });
                        if (response && response.data) {
                            emailResponse = response.data;
                            this.log.info("Nitional Service Email Response", emailResponse);
                        }
                        else {
                            this.log.error("Unable to get response from nitinol email service");
                            emailResponse = {
                                error: "Unable to get response from nitinol email service",
                            };
                        }
                    }
                    catch (err) {
                        this.log.error("Error occurred in sending email", err);
                        return { error: "Error occurred in sending email" };
                    }
                }
                else {
                    this.log.error("Unable to get JWT token response from nitinol");
                    emailResponse = {
                        error: "Unable to get JWT token response from nitinol",
                    };
                }
            }
            catch (err) {
                this.log.error("Error in fetching JWT token" + err);
                return { error: "Error in fetching JWT token" };
            }
            return emailResponse;
        });
    }
}
exports.NitinolService = NitinolService;
NitinolService.log = index_1.LambdaLogger.getInstance("NitinolService");
function getSecretValue(secretKeyName, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = {
            endpoint: "https://secretsmanager.us-east-1.amazonaws.com",
            region: "us-east-1",
        };
        const secretsClient = new aws_sdk_1.SecretsManager(options);
        const params = {
            SecretId: secretKeyName,
        };
        try {
            const data = yield secretsClient.getSecretValue(params).promise();
            if (data) {
                if (data.SecretString) {
                    return data.SecretString;
                }
                else {
                    return data.SecretBinary;
                }
            }
            else {
                return false;
            }
        }
        catch (error) {
            logger.error("Failed to Retrieve the secret value: " + error);
            return false;
        }
    });
}
